/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

	File:	messiah_shader.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 12/01/04 - LM
			- fxShaderSpotRotate
			- fxShaderSpotScale
			- fxShaderSpotXForm
			- fxRenderSettings
		last modified: 06/25/04 - LM
			- fxShaderGlobalsLight
			- fxShaderRenderCheck
			- fxGIType
			- fxBufferImageSize

		last modified: 03/14/04 - LM
			- changed all references from FX_Point to FX_Vector.  This is done
			in order to avoid datatype collisions.
			- added functions for raytracing/casting & illumination



	-----------------------------------------------------------------
	INPUTS/OUTPUTS
	-----------------------------------------------------------------
	When you develop a messiah shader, you have the ability to create
	inputs & outputs.  These inputs & ouputs are displayed on the
	shadernode in the shaderflow in Render mode, allowing the user to
	connect your shader to other shaders and/or the material.  It's
	through the inputs that you'll receive data to "evaluate" your
	shader, and it's through the outputs that you'll send the results
	of that evaluation.  The "IO" can represent any data you like (as
	long as you use the FX_IOTYPE types), and need not correspond to
	motion channels in your shader.

	EX1: Creating Inputs & Outputs.

		case O_CREATE:
		{
			.
			.
			.

			IN_FOG_COLOR		= fxShaderCreateInput(shader, "Color", FX_IOTYPE_COLOR, 0);
			OUT_FOG_RADIANCE	= fxShaderCreateOutput(shader, "Fog_Radiance", FX_IOTYPE_COLOR, 0);


	As you can see, an ID will be returned for each IO.  You'll use
	this ID to get/set info in your ShaderInitInputs and ShaderEvaluate
	callbacks:

	EX2:  Initializing inputs

		FX_SHADERINITINPUTS(ShaderInitInputs)
		{
			My_ShaderData	*msd = fxShaderGetTypeData(shader, 0);	.

			input[IN_FOG_COLOR].color.r = (FXfloat)msd->color.r;
			input[IN_FOG_COLOR].color.g = (FXfloat)msd->color.g;
			input[IN_FOG_COLOR].color.b = (FXfloat)msd->color.b;

			return;
		}

	EX3:  Evaluating outputs

		FX_SHADERINITINPUTS(ShaderEvaluate)
		{
			My_ShaderData	*msd = fxShaderGetTypeData(shader, 0);.

			output[OUT_FOG_RADIANCE].color.r = input[IN_FOG_COLOR].color.r * (FXfloat)_N.y;
			output[OUT_FOG_RADIANCE].color.g = input[IN_FOG_COLOR].color.g * (FXfloat)_N.y;
			output[OUT_FOG_RADIANCE].color.b = input[IN_FOG_COLOR].color.b * (FXfloat)_N.y;

			return;
		}

	In EX2, the created input is initialized with custom data.  Input 
	initialization is an important step because IO compositing requires
	a base value.  So, always initialize your inputs.

	In EX3, the input is used with global data to compute the output.
	The input may or may not be the same value to which you initialized
	it in InitInputs.  There may be other shaders attatched to that
	input and may change the value.  This is the power of the system.

	Note that because shaders can have up to 31 outputs (32, actually
	since opacity IO are created by default), you may not be required
	to compute all of your inputs.  So, you should perform a check
	before computing a particular output.

	EX4:  Evaluating requested outputs

		FX_SHADERINITINPUTS(ShaderEvaluate)
		{
			My_ShaderData	*msd = fxShaderGetTypeData(shader, 0);.

			if (FX_OUTPUT_REQUEST(OUT_FOG_RADIANCE))
			{
				output[OUT_FOG_RADIANCE].color.r = input[IN_FOG_COLOR].color.r * (FXfloat)_N.y;
				output[OUT_FOG_RADIANCE].color.g = input[IN_FOG_COLOR].color.g * (FXfloat)_N.y;
				output[OUT_FOG_RADIANCE].color.b = input[IN_FOG_COLOR].color.b * (FXfloat)_N.y;
			}

			return;
		}


	Normally, all of your inputs will be pre-processed for you, i.e
	any shaders connected to your inputs will have already been
	processed and composited by the time your Evaluate callback is
	executed.  However, there may be times when you need to process
	the inputs yourself.  This will be the case if you're, say,
	creating a volumetric shader that needs to get an input color for
	spots other than the current hit.  First you'd need to setup the
	input so that it doesn't automatically process using the
	FX_SIO_NOPROC option.

	EX5: Creating non-auto processing inputs.

		case O_CREATE:
		{
			.
			.
			.

			IN_FOG_COLOR		= fxShaderCreateInput(shader, "Color", FX_IOTYPE_COLOR, FX_SIO_NOPROC);


	Then you'd have to process the inputs yourself inside your 
	Evaluate callback.

	EX6:  Evaluating requested outputs & processing inputs

		FX_SHADERINITINPUTS(ShaderEvaluate)
		{
			My_ShaderData	*msd = fxShaderGetTypeData(shader, 0);.
			FXint			connected = 0;

			fxShaderConnectedInputs(shader, &connected, 0);

			if (FX_OUTPUT_REQUEST(OUT_FOG_RADIANCE))
			{
				if (connected & (1<<IN_FOG_COLOR))
				{
					fxShaderProcessInput(shader, shader_globals, IN_FOG_COLOR, &input[IN_FOG_COLOR], 0);
				}

				output[OUT_FOG_RADIANCE].color.r = input[IN_FOG_COLOR].color.r * (FXfloat)_N.y;
				output[OUT_FOG_RADIANCE].color.g = input[IN_FOG_COLOR].color.g * (FXfloat)_N.y;
				output[OUT_FOG_RADIANCE].color.b = input[IN_FOG_COLOR].color.b * (FXfloat)_N.y;
			}

			return;
		}

	You'll note that in EX6 we're calling fxShaderConnectedInputs.
	This is the function you'll need to use to determine if an input
	has any connected shaders.  If there are no connected shaders to
	a particular input, then there is no need to process that input.

	NOTE: Since the connections will not change during a render, there
	really is no need to call fxShaderConnectedInputs inside of your
	Evaluate callback.  It would be more efficient to create a
	"connected" member inside your shader's data struct, and perform
	the fxShaderConnectedInputs inside your Init callback.  It was
	written in the less efficient manner here for illustrative
	purposes only.
	-----------------------------------------------------------------



	-----------------------------------------------------------------
	TRACING OR CASTING RAYS
	-----------------------------------------------------------------
	In your FX_SHADEREVALUATE callcack, there may be times when you
	want to trace rays to get back a color for a hit, or to cast rays
	to determine if something intersects the ray.  The API provides a
	very simple way of performing these operations.

	EX1: Casting A Ray to Determine if there is an intersection.

		//Note: use the func in this way if you don't require any other
		//info beyond hit detection for a ray.

		FXint			hit = FX_FALSE;

		hit = fxRayCast(shader_globals, FX_RAYTYPE_GENERIC, _P, direction, NULL, NULL, NULL, 0);

		if (hit)
		{
			//Do something based on the hit, if needed
			//You can use the fxIlluminate funcs if needed
		}


	EX2: Casting A Ray to get an object hit

		//Note: if you require object data, or any other info for the hit,
		//you must create a ray hit pointer with the provided func.

		FX_RayHit		*rh = NULL;
		FXint			hit = FX_FALSE;

		rh = fxRayHitCreate(0);
		hit = fxRayCast(shader_globals, FX_RAYTYPE_GENERIC, _P, direction, rh, NULL, NULL, 0);

		if (hit)
		{
			fxRayHitObject(rh, &obj, NULL, &poly, NULL, 0);

			//Do something based on the object returned.
			//You can use the fxObject and/or fxMeshPoly funcs if needed
			//You can use the fxIlluminate funcs if needed
		}

		//Once you're done with the ray hit data, you must free it
		fxRayHitDelete(rh, 0);

	EX3: Casting A Ray to get spot info

		FX_RayHit		*rh = NULL;
		FXint			hit = FX_FALSE;
		FXdouble		ray_length;
		FXVector		pos;
		FXVectorF		normal;

		rh = fxRayHitCreate(0);
		hit = fxRayCast(shader_globals, FX_RAYTYPE_GENERIC, _P, direction, rh, NULL, NULL, 0);

		if (hit)
		{
			fxRayHitSpot(rh, &ray_length, &pos, &normal, NULL, 0);

			//Do something with the spot.
			//You can use the fxIlluminate funcs if needed
		}

		//Once you're done with the ray hit data, you must free it
		fxRayHitDelete(rh, 0);

	EX4: Tracing A Ray to get surface color

		FX_RayHit		*rh = NULL;
		FXint			hit = FX_FALSE;
		FX_Color		radiance;
		FXfloat			alpha;

		rh = fxRayHitCreate(0);
		hit = fxRayTrace(shader_globals, FX_RAYTYPE_GENERIC, _P, direction, 0, 0, rh, NULL, NULL, 0);

		if (hit)
		{
			fxRayHitColor(rh, &radiance, NULL, NULL, NULL, 0);
			fxRayHitAlpha(rh, &alpha, 0);

			//Do something with the color & alpha, as needed.
			//You can use the fxIlluminate funcs if needed
		}

		//Once you're done with the ray hit data, you must free it
		fxRayHitDelete(rh, 0);


	As you can see, when you trace or cast rays, and need to get
	specific data in regards to that hit (if any), you must supply an
	FX_RayHit pointer.  This is an opaque datatype, so you'll need to
	use the fxRayHitCreate to allocate the pointer.  You can allocate
	as many ray hit pointers as needed, and you can reuse them as
	necessary.  Once allocated, the ray hit pointer is *your* data.
	This means that you're also responsible for freeing the pointer
	when you no longer need it, via the fxRayHitDelete func.
	-----------------------------------------------------------------


	-----------------------------------------------------------------
	GEOMETRY TYPES
	-----------------------------------------------------------------
	messiah has internal geometry types to which your shader may be
	attatched (via its main Material).  When your shader's evaluate
	callback is called, if needed you can determine the type of
	geometry by retreiving it from shader_globals.

	EX1: Getting the polygon type from shader_globals

	FXobject	obj;
	FXint		polyID;
	FXint		geom_type;

	fxShaderGlobalsObject(shader_globals, &obj, NULL, &polyID, &geom_type, 0);

	if (geom_type==FX_GEOMTYPE_POLYGON)
	{
	}



------------------------------------------------------------------
HERE'S AN EXAMPLE OF HOW TO USE IT TO STEP BACK AND ACCUM THE RAY LENGTH
 
//steping backward example
FX_ShaderGlobals *sg_current=shader_globals;
double accm_length=0.0;
while(sg_current)
{
	//get ray length from sg_current

	accm_length+=sg_current->Rl;
	///////////////////////////////////step back

	fxPrevShaderGlobals(sg_current, &sg_current,0);
}

=============================================================================*/


#ifdef _MESSIAH_API_SHADER

#ifndef __MESSIAH_SHADER
#define __MESSIAH_SHADER


#ifndef __MESSIAH_HIDE_TYPES


	#define MAX_NUM_THREADS				8
 
	//Shader Callback types----------------------------------------------------
	#define FX_SHADERFUNC_INIT			0
	#define FX_SHADERFUNC_INITINPUTS	1
	#define FX_SHADERFUNC_NEWTIME		2
	#define FX_SHADERFUNC_EVALUATE		3
	#define FX_SHADERFUNC_DONE			4
	#define FX_SHADERFUNC_IMAGEFILTER	5
	//`````````````````````````````````````````````````````````````````````````

	//Shader Input/Output types------------------------------------------------
	#define FX_IOTYPE_FLOAT				1
	#define FX_IOTYPE_COLOR				2
	#define FX_IOTYPE_VECTOR			3
	#define FX_IOTYPE_POINT				4
	#define FX_IOTYPE_INT				5
	#define FX_IOTYPE_STRING			6
	#define FX_IOTYPE_VOID				7
	//`````````````````````````````````````````````````````````````````````````

	//Shader Input/Output flags------------------------------------------------
	#define FX_SIO_NOPROC				(1<<31) //input will not be processed automatically.  You'll process it yourself using fxShaderProcessInput
	//`````````````````````````````````````````````````````````````````````````

	//Macro for determining if you should compute an output-------------------
	#define	FX_OUTPUT_REQUEST(o)		(_out_req & (1<<o))
	//`````````````````````````````````````````````````````````````````````````

	//Macro for declaring Input/Output ID FXints------------------------------
	#define FX_IO_ID(v)					FX_VAR_I(FXint, v, 0)
	//`````````````````````````````````````````````````````````````````````````

	//Arbitrary data type for inputs/outputs & modifiable shader "globals"-----
	#define FX_POINT	FX_ShaderParam
	#define FX_COLOR	FX_ShaderParam
	#define FX_COLOR4	FX_ShaderParam
	#define FX_VECTOR	FX_ShaderParam
	#define FX_VOID		FX_ShaderParam
	#define FX_REAL		FX_ShaderParam
	#define FX_DOUBLE	FX_ShaderParam
	#define FX_FLOAT	FX_ShaderParam
	#define FX_STRING	FX_ShaderParam
	#define FX_INT		FX_ShaderParam

	typedef union FX_ShaderParam_Un
	{
		FXint		i;
		FXfloat		f;
		FXchar		str[64];
		FX_Color	color;
		FX_Vector	vec;
		FX_Vector	pnt;
		FXvoid		*v;

	}FX_ShaderParam;
	//`````````````````````````````````````````````````````````````````````````


	//Developer Modifiable shader "globals"------------------------------------
	typedef struct FX_ModShaderGlobals_St
	{              
		FX_POINT		P;				// Shading point                              
		FX_VECTOR		N;				// Shading normal                            
		FX_VECTOR		Ng;				// Geometric normal                           
		FX_FLOAT		u;				// UV surface parameters                      
		FX_FLOAT		v;				//                                           
		FX_VECTOR		Ld;				// Incident direction from light source       
		FX_COLOR		Li;				// Incoming intensity from light source       
		FX_COLOR		Cs;				// Input color                                
		FX_COLOR		Co;				// Output color                               

		FX_COLOR		Os;				// Input Opacity							
		FX_COLOR		Oo;				// Ouput Opacity							

		FX_COLOR		Mo;				// Output diffuse radiance (Co=Cs*Mo)         
		FX_COLOR		No;				// Output specular radiance (Co=Cs*Mo+No)     
		FX_COLOR		Vo;				// Output volume radiance (Co=Cs*Mo+No+Vo)
		
		FX_FLOAT		s_off;
		FX_FLOAT		t_off;

		FX_FLOAT		u_off;
		FX_FLOAT		v_off;

		FX_INT			isec_state;

		FX_POINT		Pl;				// Local shading point (P -> object coordinates)

		FX_REAL			Ll;				// Light ray length (light distance to hit)

		FX_VOID			dont_modify[3];	// ***FOR INTERNAL USE***

	}FX_ModShaderGlobals;
	//`````````````````````````````````````````````````````````````````````````


	//Shader "globals" data structure------------------------------------------
	typedef struct FX_ShaderGlobals_St
	{
		FX_ModShaderGlobals	mod;				//contains all the modifiable globals that shaders can change, that are commented above

		FXint				x;					// Pixel that spawned this ray                
		FXint				y;					//                                            
		FXdouble			xf;					// Screen location of this ray [0..1]         
		FXdouble			yf;					//                                            
		FXint				si;					// Subpixel index                             

		FXint				render_stage;
		FXint				output_request;		// bits representing the output channel being requested of the current shader
		FX_Vector			P_orig;				// Original Shading point                              
		FX_Vector			N_orig;				// Original Shading normal                             
		FX_Color			Cs_orig;			// Original Input color                                
		FX_Color			Os_orig;			// Original Opacity                           
		FXdouble			ineta;				// incoming refraction index
		FXfloat				translu;			// ***FOR INTERNAL USE***
		FXfloat				Kt;
		FXfloat				volume_factor;		// ***FOR INTERNAL USE***
		FXdouble			transm_range;		// ***FOR INTERNAL USE***
		FXdouble			transm_falloff;		// ***FOR INTERNAL USE***
		FXint				hit_back;
		FXint				transm_potential;	// ***FOR INTERNAL USE***
		FXfloat				spot_size;			// size of the hit
		FXfloat				spot_aspect;		// aspect ratio of the hit (width = spot_size, height = spot_size*spot_aspect, depth = spot_size)
		FXfloat				surface_flex;		// the amount of area distortion **FILLED IN ONLY IF fxShaderSurfaceFlex HAS BEEN CALLED**

		FXint				RrD;				// diffuse recursion ray level
		FXint				RrS;				// specular recursion ray level

		FXint				Rt;					// Ray type                                   
		FX_Vector			Ro;					// Ray origin (typically the camera)          
		FX_Vector			Rd;					// Ray direction                              
		FXint				Rr;					// Ray recursion level                        
		FXdouble			Rl;					// Ray length (|Ro-P|)                        
		FX_Vector			Nf;					// ***FOR INTERNAL USE***
		FXfloat				NdotV;				// ***FOR INTERNAL USE***
		FXfloat				Fr;					// ***FOR INTERNAL USE***

		FXint				tid;				// Thread ID   
		
		FXbool				photon;				// say wether this is a photon type shade
		FXfloat				photon_power[3];
		FXchar				photon_type;

		FXdouble			time;

		FXfloat				alpha;
		FXint				hit_backDS;			//back hit indicator for double sided objects
		FXint				INTERNAL1;
		FXfloat				INTERNAL2;
		FXfloat				INTERNAL3;
		FX_Color			Opacity;			//
		FXchar				dont_modify[32];	// ***FOR INTERNAL USE***

	}FX_ShaderGlobals;
	//`````````````````````````````````````````````````````````````````````````
	//Particle Info Stucture------------------------------------------

typedef struct FX_ParticleHit_St{
	   FXint ID;
	   FXdouble t;
		FX_Vector hit_pos;
		FXfloat u;
		FXfloat v;
	}FX_ParticleHit;


	//Shader callback protope macros-------------------------------------------
	#define FX_SHADERINIT(f)		FXvoidf f(FXshader shader)
	#define FX_SHADERNEWTIME(f)		FXvoidf f(FXshader shader)
	#define FX_SHADERINITINPUTS(f)	FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input)
	#define FX_SHADEREVALUATE(f)	FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input, FX_ShaderParam *output)
	#define FX_SHADERDONE(f)		FXvoidf f(FXshader shader)
	#define FX_SHADERFILTER(f)		FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input, FXint posx, FXint posy, FXint width, FXint height)
	//`````````````````````````````````````````````````````````````````````````


	//Pre-defined shader inputs/outputs----------------------------------------
	#define IN_OPACITY		0
	#define OUT_OPACITY		0
	#define _in_opacity		input[IN_OPACITY].f
	#define _out_opacity	output[OUT_OPACITY].f
	//`````````````````````````````````````````````````````````````````````````

	#ifndef __MESSIAH_HIDE_SG_DEFINES
		//Shader Global defines----------------------------------------------------
		//Modifiable globals,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		#define _P					(shader_globals->mod.P.pnt)
		#define _Pl					(shader_globals->mod.Pl.pnt)
		#define _N					(shader_globals->mod.N.vec)
		#define _Ng					(shader_globals->mod.Ng.vec)
		#define _u					(shader_globals->mod.u.f)
		#define _v					(shader_globals->mod.v.f)
		#define _Ld					(shader_globals->mod.Ld.vec)
		#define _Li					(shader_globals->mod.Li.color)
		#define _Cs					(shader_globals->mod.Cs.color)
		#define _Co					(shader_globals->mod.Co.color)
		#define _Os					(shader_globals->mod.Os.color)									
		#define _Oo					(shader_globals->mod.Oo.color)
		#define _Mo					(shader_globals->mod.Mo.color)
		#define _No					(shader_globals->mod.No.color)
		#define _Vo					(shader_globals->mod.Vo.color)
		#define _s_off				(shader_globals->mod.s_off.f)
		#define _t_off				(shader_globals->mod.t_off.f)
		#define _Ll					(shader_globals->mod.Ll.f)

		//Constant globals,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		#define _Nf					(shader_globals->Nf)
		#define _NdotV				(shader_globals->NdotV)
		#define _Fr					(shader_globals->Fr)
		#define _Rt					(shader_globals->Rt)
		#define _Ro					(shader_globals->Ro)
		#define _Rd					(shader_globals->Rd)
		#define _Rl					(shader_globals->Rl)
		#define _Rr					(shader_globals->Rr)
		#define _x					(shader_globals->x)
		#define _y					(shader_globals->y)
		#define _xf					(shader_globals->xf)
		#define _yf					(shader_globals->yf)
		#define _si					(shader_globals->si)
		#define _tid				(shader_globals->tid)
		#define _transm_potential	(shader_globals->transm_potential)
		#define _transm_range		(shader_globals->transm_range)
		#define _transm_falloff		(shader_globals->transm_falloff)

		#define _spot_size			(shader_globals->spot_size)
		#define _hit_back			(shader_globals->hit_back)
		#define _render_stage		(shader_globals->render_stage)
		#define	_P_orig				(shader_globals->P_orig)                                    
		#define	_N_orig				(shader_globals->N_orig)                                  
		#define	_Cs_orig			(shader_globals->Cs_orig)                                     
		#define	_Os_orig			(shader_globals->Os_orig)                               
		#define	_ineta				(shader_globals->ineta)
		#define _time				(shader_globals->time)
		#define _RrD				(shader_globals->RrD)
		#define _RrS				(shader_globals->RrS)
		#define _alpha				(shader_globals->alpha)
		#define _Opacity			(shader_globals->Opacity)

		#define _out_req			(shader_globals->output_request)
		//``````````````````````````````````````````````````````````````````````
	#endif //__MESSIAH_HIDE_SG_DEFINES


	//Datatype for Illumination funcs,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	typedef struct FX_SamplePattern_St	FX_SamplePattern;

	//Daytypes for RayTrace, RayCast, & RayHit funcs,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	typedef struct FX_Ray_St			FX_Ray;
	typedef struct FX_RayHit_St			FX_RayHit;

	//Datatype for Buffer & BufferSample funcs (image filters) ,,,,,,,,,,,,,,,,,
	typedef struct FX_BufferSample_St	FX_BufferSample;


	//Ray types,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_RAYTYPE_CAMERA				0x00
	#define FX_RAYTYPE_SHADOW				0x01
	#define FX_RAYTYPE_REFLECTION			0x02
	#define FX_RAYTYPE_DIFFUSE_REFLECTION	0x03
	#define FX_RAYTYPE_REFRACTION			0x04
	#define FX_RAYTYPE_GENERIC				0x05

	//Side preference for use fxRaySidePref/Set,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	//Tells a ray what side of a polygon you're looking to hit.
	#define FX_RAYHIT_SIDE_DEFAULT			0 //observes user polygon settings of double sided or single sided
	#define FX_RAYHIT_SIDE_FRONT			1 // will only return a hit if it hits the front side
	#define FX_RAYHIT_SIDE_BACK				2 // will only return a hit if it hits the back side
	#define FX_RAYHIT_SIDE_ANY				3 // Will return hit if it hits any side (front or back)

	//set criteria for rays tests based on the item currently being shaded
	//USE PREDEFINES BELOW
	#define FX_RHNOT		(1<<1)
	#define FX_RHONL		(1<<2)
	#define FX_RHOBJ		(1<<5)
	#define FX_RHGRP		(1<<6)
	#define FX_RHSRF		(1<<7)
	//pre defines for above combinations
	#define FX_RAYHIT_ANYOBJ			0
	#define FX_RAYHIT_NOTME_OBJ			(FX_RHNOT|FX_RHOBJ)
	#define FX_RAYHIT_NOTME_GROUP		(FX_RHNOT|FX_RHGRP)
	#define FX_RAYHIT_NOTME_SURFACE		(FX_RHNOT|FX_RHSRF)
	#define FX_RAYHIT_ONLYME_OBJ		(FX_RHONL|FX_RHOBJ)
	#define FX_RAYHIT_ONLYME_GROUP		(FX_RHONL|FX_RHGRP)
	#define FX_RAYHIT_ONLYME_SURFACE	(FX_RHONL|FX_RHSRF)
	///////////////////////////////////////////////
	
	
	//Geometry types,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_GEOMTYPE_POLYGON				(1<<7)
	#define FX_GEOMTYPE_SUBPOLYGON			(1<<10)
	#define FX_GEOMTYPE_BLOB				(1<<12)
	#define FX_GEOMTYPE_PARTICLE			(1<<13)


	//SetRayData callback macro definition for use with fxRayCast/Trace,,,,,,,,,
	#define FX_RAYDATAFUNC(f, dt, data)		FXintf f(FX_ShaderGlobals *shader_globals, FX_Ray *ray, dt *data)


	//Filter flags for use with fxRayCast,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	//**NOT CURRENTLY ACTIVE**
	#define FX_RAYFILTER_NOSHADOW_SELF		(1<<2)
	#define FX_RAYFILTER_NOSHADOW_CAST		(1<<3)
	#define FX_RAYFILTER_NOSHADOW_RECEIVE	(1<<4)
	#define FX_RAYFILTER_NOREFLECT_CAST		(1<<5)
	#define FX_RAYFILTER_NOSHADOW			(FX_RAYFILTER_NOSHADOW_SELF|FX_RAYFILTER_NOSHADOW_CAST|FX_RAYFILTER_NOSHADOW_RECEIVE)
	#define FX_RAYFILTER_NOREFLECT			(FX_RAYFILTER_NOREFLECT_CAST)
	#define FX_RAYFILTER_INCLUSION			(1<<30)

	//IlluminateFunc callback macro definition for use with fxIlluminateDirect,,
	#define FX_ILLUMINATEFUNC(f, dt, data)	FXintf f(FX_ShaderGlobals *shader_globals, FXobject lightID, FX_Vector dir, FX_Color *color, FXfloat diffuse, FXfloat specular, FX_Color transm, dt *data)

	//Illumate flags,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define	FX_ILLUM_UNIFORMRAY				(1<<20) //fxIllumiateIndirect flag: ray distribution. if not set, weighted distribution is used

	//NOTE: mutually exculsive poly side preference flags. If neither of these flags are used, the default side pref is assumed.
	#define	FX_ILLUM_FRONT					(1<<25) //fxIllumiateIndirect flag: side pref. Front
	#define	FX_ILLUM_BACK					(1<<26) //fxIllumiateIndirect flag: side pref. Back
	#define	FX_ILLUM_ANY					(1<<27) //fxIllumiateIndirect flag: side pref. Any

	#define	FX_ILLUM_NOEXCLUSION			(1<<31) //surface->light exclusion is ignored during the call to fxIllumiateDirect


	//for use with fxShaderRenderCheck,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_SHADRENDER_SAMPLE			(1<<1) //currently rendering a material sample (not an output image


	//fxBufferPixel flags for setting the pixel data into shader_globals,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	//NOTE: This is useful for Image Filters that need to prepare shader_globals data
	//to process its connected input shaders.
	#define FX_BUFFGLOBALS_FRAME			(1<<0) //fill in frame coordinates
	#define FX_BUFFGLOBALS_FRAMEPOS			(1<<1) //fill in the frame coordinates as _P
	#define FX_BUFFGLOBALS_OBJECT			(1<<2) //fill in the object hit data
	#define FX_BUFFGLOBALS_COLOR			(1<<3) //fill in the color data


	//return types for fxGIType,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_GITYPE_NONE					0
	#define FX_GITYPE_MONTE					1
	#define FX_GITYPE_PHOTON				2
	#define FX_GITYPE_MONTE_PHOTON			3


	//render setting values queried from fxRenderSettingsGet,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_REN_GI_SAMPLES				0
	#define FX_REN_SPECULAR_DEPTH			1
	#define FX_REN_DIFFUSE_DEPTH			2

	//for use with fxShaderSpotRotate, fxShaderSpotScale, & fxShaderSpotXForm
	#define FX_SPOTXFORM_INVERSE	(1<<1)

	//to tell the system you want to be called for shadow rays and you may allow light to pass through you
	#define FX_SHADER_PROC_TRANSM (1<<0)

#endif //__MESSIAH_HIDE_TYPES


//=============================================================================
//SHADER SETUP & COMPUTATION
//=============================================================================

FX_API(FXint, fxShaderCreateInput)(FXshader, FXchar *name, FXint io_type, FXint flags);

FX_API(FXint, fxShaderCreateOutput)(FXshader, FXchar *name, FXint io_type, FXint flags);

FX_API(FXint, fxShaderInputSetAlias)(FXshader, FXint input, FXchar *alias, FXint flags);

FX_API(FXint, fxShaderOutputSetAlias)(FXshader, FXint output, FXchar *alias, FXint flags);

FX_API(FXint, fxShaderInputGetAlias)(FXshader, FXint input, FXchar alias[32], FXint flags);

FX_API(FXint, fxShaderOutputGetAlias)(FXshader, FXint output, FXchar alias[32], FXint flags);

FX_API(FXint, fxShaderCallback)(FXshader, FXint callback_type, FXvoid *callback);

FX_API(FXvoid*, fxShaderGetTypeData)(FXshader, FXint flags);

FX_API(FXint, fxShaderSetTypeData)(FXshader, FXvoid *data, FXint flags);

FX_API(FXint, fxShaderSpotToLocal)(FX_ShaderGlobals *, FX_Vector *world_spot, FX_Vector *local_spot, FXint flags);

FX_API(FXint, fxShaderIncidence)(FX_ShaderGlobals *, FXdouble *u, FXint flags);

FX_API(FXint, fxShaderLightSpot)(FX_ShaderGlobals *, FX_Color *light_color, FXint flags);

FX_API(FXint, fxShaderSpecular)(FX_ShaderGlobals *, FXdouble specular, FXdouble gloss, FX_Color *specular_color, FXint flags);

FX_API(FXint, fxShaderGlobalsObject)(FX_ShaderGlobals *, FXobject *, FXint *surfID, FXint *polyID, FXint *geom_type, FXint flags);

FX_API(FXint, fxShaderGlobalsSubpoly)(FX_ShaderGlobals *,  FX_Vector points[3], FX_VectorF normals[3], FX_VectorF2D uvs[3], FXint flags);

FX_API(FXint, fxShaderGlobalsRay)(FX_ShaderGlobals *, FX_Ray **ray, FXint flags);

FX_API(FXint, fxShaderGlobalsLight)(FX_ShaderGlobals *, FXobject *lightID, FXint flags);

FX_API(FXint, fxShaderGlobalsRaySenderObject)(FX_ShaderGlobals *, FXobject *objectID, FXint flags);

FX_API(FXint, fxShaderProcessInput)(FXshader, FX_ShaderGlobals *, FXint inputID, FX_ShaderParam *value, FXint flags);
FX_API(FXint, fxShaderProcessInput_SendData)(FXshader, FX_ShaderGlobals *, FXint inputID, FXdouble data, FX_ShaderParam *value, FXint flags );

FX_API(FXint, fxShaderConnectedInputs)(FXshader, FXint *connected, FXint flags);

FX_API(FXint, fxShaderRenderCheck)(FX_ShaderGlobals *, FXint flags);

FX_API(FXint, fxShaderSurfaceFlex)(FX_ShaderGlobals *, FXfloat *flex, FXint flags);//gives flex>1 for stretch and flex<1 for compression

FX_API(FXint, fxShaderGlobalsParticleID)(FX_ShaderGlobals *, FXint *particleID, FXint flags);

FX_API(FXint, fxShaderSpotRotate)(FXshader, FX_ShaderGlobals *, FXdouble hpb[3], FX_Vector pnt, FX_Vector *new_pos, FXint flags);

FX_API(FXint, fxShaderSpotScale)(FXshader, FX_ShaderGlobals *, FXdouble scale[3], FX_Vector pnt, FX_Vector *new_pos, FXint flags);

FX_API(FXint, fxShaderSpotXForm)(FXshader, FX_ShaderGlobals *, FXdouble xform[9], FX_Vector pnt, FX_Vector *new_pos, FXint flags);

FX_API(FXint, fxRenderSettings)(FXint setting, FXfloat *value, FXint flags);

FX_API(FXint, fxShaderSetProcFlags)(FXshader,FXint flags); //set flags to tell system that you want your shader to be called for shadow rays etc.

FX_API(FXint, fxShaderMetaballParticleIDCount)(FX_ShaderGlobals *sg,FXint *count,FXint flags);

FX_API(FXint, fxShaderMetaballParticleIDList)(FX_ShaderGlobals *sg,FXint *IDlist,FXint flags);

FX_API(FXint, fxShaderBlobParticleIDCount)(FX_ShaderGlobals *sg,FXint *count,FXint flags);

FX_API(FXint, fxShaderBlobParticleIDList)(FX_ShaderGlobals *sg,FXint *IDlist,FXint flags);

FX_API(FXint, fxShaderBlobParticleGetHitList)(FX_ShaderGlobals *sg,FX_ParticleHit *hits,FXint flags);

FX_API(FXint, fxShaderBlobParticleIDGetInfo)(FX_ShaderGlobals *sg,FXint ID,FX_Vector *hit_pos, FXfloat *u, FXfloat *v, FXint flags);

FX_API(FXint, fxPrevShaderGlobals)(FX_ShaderGlobals *sg_current, void **sg_prev, FXint flags);
//=============================================================================
//Atmosphere
//=============================================================================

FX_API(FXint, fxEvaluateAtmosphere)(FX_ShaderGlobals *sg,FX_Color *color,FX_Color *opacity);

//=============================================================================
//ILLUMINATION
//=============================================================================

FX_API(FXint, fxGIType)(FXvoid);

FX_API(FXvoid, fxAASamples)(FXint *aa_samples, FXint *aa_pass);

FX_API(FX_SamplePattern*, fxSamplePatternCreate)
(
	FXint		sample_quality, //this number will be squared n*n to create the actual number samples 
	FXfloat		jitter
);

FX_API(FXint, fxSamplePatternFree)(FX_SamplePattern *pattern);


///////////////////////////
//DIFFUSE MONTE CARLO
///////////////////////////

FX_API(FXint, fxIlluminateIndirect)
(
	FX_ShaderGlobals	*,
	FX_Vector			P,
	FX_Vector			N,
	FX_Vector			Ng,
	FX_SamplePattern	*pattern,
	FXint				level,			//recursion level of this ray for bounce stoping to user interface setting
	FX_Color			*light_color,
	FXfloat				cone_scale,
	FXint				flags
);

FX_API(FXint, fxIlluminateIndirectScatter)
(
	FX_ShaderGlobals	*,
	FX_Vector			P,
	FX_SamplePattern	*pattern,
	FXint				level,			//recursion level of this ray for bounce stoping to user interface setting
	FXint				scatter, //0 anisotropic, 1 hazy
	FX_Color			*light_color,
	FXvoid				*data, //data that you'd like sent to your ScatterFunc
	FXfloat				(*ScatterFunc)(FX_ShaderGlobals *, FX_Vector Rd, FX_Vector Ld, FXvoid *data), //optional user function to give perc back given a dir
	FXint				flags
);

///////////////////////////
//DIRECT LIGHTING/LOOP
////////////////////////////

FX_API(FXint, fxIlluminateDirect)
(
	FX_ShaderGlobals	*,
	FX_Vector			P,
	FX_Vector			*N, //set if you want the light to reject areas that can't be seen behind this normal
	FX_Vector			*Ng, //set if you want the light to reject areas that can't be seen behind this normal 
	FX_Color			*light_color,
	FXvoid				*data, //data that you'd like sent to your IlluminateFunc
	FXint				(*IlluminateFunc)(FX_ShaderGlobals *, FXobject lightID, FX_Vector dir, FX_Color *color, FXfloat diffuse, FXfloat specular, FX_Color transm, FXvoid *data),//optional if the user wants to be called for each light
	FXint				flags
);

FX_API(FXint, fxIlluminateGlobal)
(
	FX_ShaderGlobals	*sg,
	FX_Vector			P,
	FX_Vector			N,
	FX_Vector			Ng,
	FXint				max_bounces,
	FX_Color			*light_color,
	float				cone_scale, // 0 to 1 representing 0 to 180 dergees of spread
	FXint					flags
);

///////////////////////////
//DIFFUSE PHOTON
////////////////////////////
FX_API(FXint, fxIlluminatePhoton)
(
	FX_Vector	P,
	FX_Vector	N, //if norm is 0.0 then we've intergrating over the sphere instead of the hemisphere
	FXint		gather_count, //send 0.0 to use user setting
	FXfloat		gather_radius, //send 0.0 to use user setting
	FX_Color	*light_color,
	FXint		flags
);

/////////////////////////////////////		
//CAUSTIC PHOTON
/////////////////////////////////////
FX_API(FXint, fxIlluminateCaustics)
(
	FX_Vector	P,
	FX_Vector	N, 
	FXint		gather_count, //send 0.0 to use user setting
	FXfloat		gather_radius, //send 0.0 to use user setting
	FX_Color	*light_color,
	FXint		flags
);

/////////////////////////
//VOLUME PHOTON 
///////////////////////
FX_API(FXint, fxIlluminateVolumePhoton)
(
	FX_Vector	P,
	FX_Vector	N, //if norm is 0.0 then we've intergrating over the sphere instead of the hemisphere
	FXint		gather_count, //send 0.0 to use user setting
	FXfloat		gather_radius, //send 0.0 to use user setting
	FX_Color	*light_color,
	FXint		flags
);




//=============================================================================
//RAYTRACING & RAYCASTING
//=============================================================================

FX_API(FXint, fxRayTrace)
(
	FX_ShaderGlobals	*,
	FXint				type,
	FX_Vector			origin,
	FX_Vector			dir,
	FXint				Slevel,
	FXint				Dlevel,
	FX_RayHit			*rh,
	FXvoid				*data,
	FXvoid				(*SetRayData)(FX_ShaderGlobals *, FX_Ray *, FXvoid *),
	FXint				flags
);


FX_API(FXint, fxRayCast)
(
	FX_ShaderGlobals	*,
	FXint				type,
	FX_Vector			origin,
	FX_Vector			dir,
	FX_RayHit			*rh, //set this to NULL if you're only concerned with hit checking
	FXvoid				*data,
	FXvoid				(*SetRayData)(FX_ShaderGlobals *, FX_Ray *, FXvoid *),
	FXint				flags
);

FX_API(FXint, fxRayResetMemory)(FXint threadID, FXint flags);

//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
//NOTE: If you perform any raytracing or raycasting inside your Init or NewTime
//callbacks, you should call fxRayResetMemory after you extract your hit data
//from a call to fxRayTrace or fxRayCast.  You can even do it after tracing 
//every X number of rays. You just don't want to trace a huge amount of rays
//without performing a Reset.  This is critical to avoid low RAM problems.
//
//THIS FUNCTION SHOULD BE CALLED ONLY IF YOU'RE TRACING/CASTING IN INIT OR NEWTIME.
//NEVER CALL THIS FUNCTION IF YOU'RE TRACING/CASTING IN INITINPUTS OR EVALUATE.
//Calling fxRayResetMemory in InitInputs or Evaluate will cause render errors; 
//ray memory is already managed by messiah in that case.
//`````````````````````````````````````````````````````````````````````````````

FX_API(FX_RayHit*, fxRayHitCreate)(FXint flags);

FX_API(FXint, fxRayHitDelete)(FX_RayHit *rh, FXint flags);



FX_API(FXint, fxRayHitObject)(FX_RayHit *rh, FXobject *, FXint *surfID, FXint *polyID, FXint *poly_type, FXint flags);

FX_API(FXint, fxRayHitSubpoly)(FX_RayHit *rh,  FX_Vector points[3], FX_VectorF normals[3], FX_VectorF2D uvs[3], FXint flags);

FX_API(FXint, fxRayHitSpot)(FX_RayHit *rh, FXdouble *ray_length, FX_Vector *pos, FX_VectorF *normal, FX_VectorF *gnormal, FXint flags);

FX_API(FXint, fxRayHitUV)(FX_RayHit *rh, FXfloat *u, FXfloat *v, FXint flags);

FX_API(FXint, fxRayHitBack)(FX_RayHit *rh, FXint flags);

//fxRayTrace - specific,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

FX_API(FXint, fxRayHitColor)(FX_RayHit *rh, FX_Color *radiance, FX_Color *texture, FX_Color *diffuse, FX_Color *specular, FXint flags);

FX_API(FXint, fxRayHitAlpha)(FX_RayHit *rh, FXfloat *alpha, FXint flags);

FX_API(FXint, fxRayHitShadow)(FX_RayHit *rh, FXfloat *shadow_hash, FXint flags);


   
//SET ray data,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

FX_API(FXint, fxRayTypeSet)(FX_Ray *ray, FXint type);

FX_API(FXint, fxRayOriginSet)(FX_Ray *ray, FX_Vector origin);

FX_API(FXint, fxRayDirectionSet)(FX_Ray *ray, FX_Vector dir);

FX_API(FXint, fxRayLevelSet)(FX_Ray *ray, FXint level);

FX_API(FXint, fxRayDLevelSet)(FX_Ray *ray, FXint Dlevel);

FX_API(FXint, fxRaySLevelSet)(FX_Ray *ray, FXint Slevel);

FX_API(FXint, fxRayMaxDistSet)(FX_Ray *ray, FXdouble max_distance);

FX_API(FXint, fxRaySidePrefSet)(FX_Ray *ray, FXint side_pref);

FX_API(FXint, fxRayIDSet)(FX_Ray *ray, FXint comID);

FX_API(FXint, fxRayMinDistSet)(FX_Ray *ray, FXdouble min_dist);

FX_API(FXint, fxRayMinDistGet)(FX_Ray *ray, FXdouble *min_dist);

//FX_API(FXint, fxRayCustomOriginSet)(FX_Ray *ray, FX_Vector custom_ray_origin);

//FX_API(FXint, fxRayCustomOriginGet)(FX_Ray *ray, FX_Vector *custom_ray_origin);

FX_API(FXint, fxRayLightSet)(FX_Ray *ray, FXobject);// Shadow rays only- light that this ray is heading toward

FX_API(FXint, fxRayNoSelfHitSet)(FX_Ray *ray, FX_ShaderGlobals *sg);

FX_API(FXint, fxRayNoSelfHitClear)(FX_Ray *ray);
	
//GET ray data,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

FX_API(FXint, fxRayType)(FX_Ray *ray, FXint *type);

FX_API(FXint, fxRayOrigin)(FX_Ray *ray, FX_Vector *origin);

FX_API(FXint, fxRayDirection)(FX_Ray *ray, FX_Vector *dir);

FX_API(FXint, fxRayAxis)(FX_Ray *ray, FX_Vector *x, FX_Vector *y, FX_Vector *z);

FX_API(FXint, fxRayLevel)(FX_Ray *ray, FXint *level);

FX_API(FXint, fxRayDLevel)(FX_Ray *ray, FXint *Dlevel);

FX_API(FXint, fxRaySLevel)(FX_Ray *ray, FXint *Slevel);

FX_API(FXint, fxRayMaxDist)(FX_Ray *ray, FXdouble *max_distance);

FX_API(FXint, fxRaySidePref)(FX_Ray *ray, FXint *side_pref);

FX_API(FXint, fxRayID)(FX_Ray *ray, FXint *comID);

FX_API(FXint, fxRayThreadID)(FX_Ray *ray, FXint *tid);

FX_API(FXint, fxRayXY)(FX_Ray *ray, FXint *x, FXint *y);

FX_API(FXint, fxRayXYScreen)(FX_Ray *ray, FXfloat *x, FXfloat *y);

FX_API(FXint, fxRayTime)(FX_Ray *ray, FXdouble *time);

FX_API(FXint, fxRaySubpixelIndex)(FX_Ray *ray, FXint *subpixel_index);


//=============================================================================
//IMAGE FILTER
//=============================================================================

FX_API(FX_BufferSample*, fxBufferSampleCreate)(FXint flags);

FX_API(FXint, fxBufferSampleDelete)(FX_BufferSample *bfs, FXint flags);

FX_API(FXint, fxBufferSampleColor)(FX_BufferSample *bfs, FX_Color *radiance, FX_Color *texture, FX_Color *diffuse, FX_Color *specular, FXint flags);

FX_API(FXint, fxBufferSampleAlpha)(FX_BufferSample *bfs, FXfloat *alpha, FXint flags);

FX_API(FXint, fxBufferSampleSpot)(FX_BufferSample *bfs, FXdouble *depth, FX_Vector *pos, FX_VectorF *normal, FXint flags);

FX_API(FXint, fxBufferSampleShadow)(FX_BufferSample *bfs, FXfloat *shadow_hash, FXint flags);

FX_API(FXint, fxBufferSampleObject)(FX_BufferSample *bfs, FXobject *, FXint *surfID, FXint *polyID, FXint *poly_type, FXint flags);

FX_API(FXint, fxBufferSampleSubpoly)(FX_BufferSample *bfs,  FX_Vector points[3], FX_VectorF normals[3], FX_VectorF2D uvs[3], FXint flags);

FX_API(FXint, fxBufferChunkInfo)(FX_ShaderGlobals *, FXint *posx, FXint *posy, FXint *width, FXint *height, FXint flags);

FX_API(FXint, fxBufferPixel)(FX_ShaderGlobals *, FXint x, FXint y, FX_BufferSample *bfs, FXint flags);

FX_API(FXint, fxBufferColorSet)(FX_ShaderGlobals *, FXint x,FXint y, FX_Color color, FXint flags);

FX_API(FXint, fxBufferAlphaSet)(FX_ShaderGlobals *, FXint x,FXint y, FXfloat alpha, FXint flags);

FX_API(FXint, fxBufferDepthSet)(FX_ShaderGlobals *, FXint x,FXint y, FXfloat depth, FXint flags);

FX_API(FXint, fxBufferImageSize)(FX_ShaderGlobals *, FXint *width, FXint *height, FXint flags);



#define _MESSIAH_PTR_SHADER		messiah_apicomponent = FX_APIFUNCS_SHADER;\
								FX_PTR(fxShaderCreateInput);\
								FX_PTR(fxShaderCreateOutput);\
								FX_PTR(fxShaderInputSetAlias);\
								FX_PTR(fxShaderOutputSetAlias);\
								FX_PTR(fxShaderInputGetAlias);\
								FX_PTR(fxShaderOutputGetAlias);\
								FX_PTR(fxShaderCallback);\
								FX_PTR(fxShaderGetTypeData);\
								FX_PTR(fxShaderSetTypeData);\
								FX_PTR(fxShaderSpotToLocal);\
								FX_PTR(fxShaderIncidence);\
								FX_PTR(fxShaderLightSpot);\
								FX_PTR(fxShaderSpecular);\
								FX_PTR(fxShaderGlobalsObject);\
								FX_PTR(fxShaderGlobalsSubpoly);\
								FX_PTR(fxShaderGlobalsRay);\
								FX_PTR(fxShaderGlobalsLight);\
								FX_PTR(fxShaderGlobalsRaySenderObject);\
								FX_PTR(fxShaderProcessInput);\
								FX_PTR(fxShaderConnectedInputs);\
								FX_PTR(fxShaderRenderCheck);\
								FX_PTR(fxShaderSpotRotate);\
								FX_PTR(fxShaderSpotScale);\
								FX_PTR(fxShaderSpotXForm);\
								FX_PTR(fxShaderSurfaceFlex);\
								FX_PTR(fxShaderGlobalsParticleID);\
								FX_PTR(fxShaderSetProcFlags);\
								FX_PTR(fxShaderMetaballParticleIDCount);\
								FX_PTR(fxShaderMetaballParticleIDList);\
								FX_PTR(fxShaderBlobParticleIDCount);\
								FX_PTR(fxShaderBlobParticleIDList);\
								FX_PTR(fxShaderBlobParticleGetHitList);\
								FX_PTR(fxShaderBlobParticleIDGetInfo);\
								FX_PTR(fxPrevShaderGlobals);\
								\
								FX_PTR(fxEvaluateAtmosphere);\
								\
								FX_PTR(fxRenderSettings);\
								FX_PTR(fxGIType);\
								FX_PTR(fxAASamples);\
								FX_PTR(fxSamplePatternCreate);\
								FX_PTR(fxSamplePatternFree);\
								FX_PTR(fxIlluminateIndirect);\
								FX_PTR(fxIlluminateIndirectScatter);\
								FX_PTR(fxIlluminateDirect);\
								FX_PTR(fxIlluminateGlobal);\
								FX_PTR(fxIlluminatePhoton);\
								FX_PTR(fxIlluminateCaustics);\
								FX_PTR(fxIlluminateVolumePhoton);\
								\
								FX_PTR(fxRayTrace);\
								FX_PTR(fxRayCast);\
								FX_PTR(fxRayResetMemory);\
								FX_PTR(fxRayHitCreate);\
								FX_PTR(fxRayHitDelete);\
								FX_PTR(fxRayHitObject);\
								FX_PTR(fxRayHitSubpoly);\
								FX_PTR(fxRayHitSpot);\
								FX_PTR(fxRayHitUV);\
								FX_PTR(fxRayHitBack);\
								FX_PTR(fxRayHitColor);\
								FX_PTR(fxRayHitAlpha);\
								FX_PTR(fxRayHitShadow);\
								FX_PTR(fxRayTypeSet);\
								FX_PTR(fxRayOriginSet);\
								FX_PTR(fxRayDirectionSet);\
								FX_PTR(fxRayLevelSet);\
								FX_PTR(fxRayDLevelSet);\
								FX_PTR(fxRaySLevelSet);\
								FX_PTR(fxRayMaxDistSet);\
								FX_PTR(fxRaySidePrefSet);\
								FX_PTR(fxRayLightSet);\
								FX_PTR(fxRayIDSet);\
								FX_PTR(fxRayMinDistSet);\
								FX_PTR(fxRayMinDistGet);\
								FX_PTR(fxRayType);\
								FX_PTR(fxRayOrigin);\
								FX_PTR(fxRayDirection);\
								FX_PTR(fxRayAxis);\
								FX_PTR(fxRayLevel);\
								FX_PTR(fxRayDLevel);\
								FX_PTR(fxRaySLevel);\
								FX_PTR(fxRayMaxDist);\
								FX_PTR(fxRaySidePref);\
								FX_PTR(fxRayID);\
								FX_PTR(fxRayThreadID);\
								FX_PTR(fxRayXY);\
								FX_PTR(fxRayXYScreen);\
								FX_PTR(fxRayTime);\
								FX_PTR(fxRaySubpixelIndex);\
								FX_PTR(fxRayNoSelfHitSet);\
								FX_PTR(fxRayNoSelfHitClear);\
								\
								FX_PTR(fxBufferSampleCreate);\
								FX_PTR(fxBufferSampleDelete);\
								FX_PTR(fxBufferSampleColor);\
								FX_PTR(fxBufferSampleAlpha);\
								FX_PTR(fxBufferSampleSpot);\
								FX_PTR(fxBufferSampleShadow);\
								FX_PTR(fxBufferSampleObject);\
								FX_PTR(fxBufferSampleSubpoly);\
								FX_PTR(fxBufferChunkInfo);\
								FX_PTR(fxBufferPixel);\
								FX_PTR(fxBufferColorSet);\
								FX_PTR(fxBufferAlphaSet);\
								FX_PTR(fxBufferDepthSet);\
								FX_PTR(fxBufferImageSize);\
								FX_PTR(fxShaderProcessInput_SendData)

#endif //__MESSIAH_SHADER

#endif //_MESSIAH_API_SHADER

